//
//  WDInstallProcessController.h
//  Installer
//
//  Created by Clark Jackson on 12/9/10.
//  Copyright 2010 Western Digital. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "WDAuthorizedTask.h"

APPKIT_EXTERN NSString* const WDHandleLiveFileServicesUninstallKey;
APPKIT_EXTERN NSString* const WDHandleLiveFileServicesInstallKey;
APPKIT_EXTERN NSString* const WDHandleLiveFileUninstallKey;
APPKIT_EXTERN NSString* const WDHandleLiveFileInstallKey;
APPKIT_EXTERN NSString* const WDHandleWD2goLiveFileUninstallerInstallKey;
APPKIT_EXTERN NSString* const WD2goLiveFileServicesBundleIdentifier;
APPKIT_EXTERN NSString* const WD2goLiveFileServicesAppName;
APPKIT_EXTERN NSString* const WD2goLiveFileNameKey;
APPKIT_EXTERN NSString* const WD2goLiveFileExecutableKey;
APPKIT_EXTERN NSString* const WD2goLiveFileKillKey;
APPKIT_EXTERN NSString* const WDLaunchWD2goLiveFileKey;
APPKIT_EXTERN NSString* const WDHandleAddingWD2goLiveFileToAutoLaunchListKey;
APPKIT_EXTERN NSString* const WDAppSupportPathKey;
APPKIT_EXTERN NSString* const WDHandle64BitKEXTInstallKey;
APPKIT_EXTERN NSString* const WDRemovePathsKey;
APPKIT_EXTERN NSString* const WDChangeOwnershipOfInstallerKey;
APPKIT_EXTERN NSString* const WDHandle64BitKEXTInstallKey;
APPKIT_EXTERN NSString* const WDHandleKEXTInstallKey;
APPKIT_EXTERN NSString* const WDHandleKEXTUninstallKey;
APPKIT_EXTERN NSString* const WDHandle64BitKEXTUninstallKey;
APPKIT_EXTERN NSString* const WDHandleRemovingComponentsKey;
APPKIT_EXTERN NSString* const WDHandleInstallingComponentsKey;
APPKIT_EXTERN NSString* const WDChangeNameOfInstallersKey;
APPKIT_EXTERN NSString* const WDPathToInstalledUninstallerKey;
APPKIT_EXTERN NSString* const WDPathToInstalledInstallerKey;
APPKIT_EXTERN NSString* const WDInstallerNameKey;
// APPKIT_EXTERN NSString* const WDPathToInstalledAppKey;
APPKIT_EXTERN NSString* const WDResourcesInstallDestinationDirPathKey;
APPKIT_EXTERN NSString* const WDBillOfMaterialsInstallFromPathsKey;
APPKIT_EXTERN NSString* const WDBillOfMaterialsInstallToPathsKey;
APPKIT_EXTERN NSString* const WDLaunchDaemonsPathKey;
APPKIT_EXTERN NSString* const WDServicesBundleIdentifierKey;

@protocol ApolloProtocol<NSObject>

- (void) doStateChange:(int)state;
- (void) presentThreadedInstallProgress;
- (void) setInstallProgress:(double) progress;
- (BOOL) bypassAuthorizationDialogFlag;
- (void) finishInstallByFramework;
- (void) finishUninstallByFramework;
- (void) cancelInstallByFramework;
- (void) cancelUninstallByFramework;

@end



@interface WDInstallProcessController : WDAuthorizedTask<ApolloProtocol>
{
    NSDictionary* processControlDict;
    BOOL shutdownServices;
    BOOL installCanceled;
    id callerController;
    BOOL forceFailure;
    
}

@property (readwrite, copy) NSDictionary* processControlDict;
@property (readwrite, retain) id callerController;
@property (readwrite, assign) BOOL installCanceled;
@property (readwrite, assign) BOOL forceFailure;

- (id) initWithPlanDict:(NSDictionary*)installDict;
- (void) setupInstallDict:(NSDictionary*)installDict;
- (void) threadedSmartWareInstall:(NSNumber *)type;
- (void) threadedSmartWareUninstall:(NSNumber *)type;
- (void) installWDQuickView;
- (void) uninstallWDQuickView;
- (void) installWDSync;
- (void) installWDSyncLaunchd;
- (void) uninstallWDSyncLaunchd;
- (void) uninstallWDSync;
- (void) unloadDaemon:(WDDaemonType)which;
- (void) loadDaemon:(WDDaemonType)which;
- (void) unloadAgent:(WDDaemonType)which;
- (void) loadAgent:(WDDaemonType)which;
- (BOOL) shouldUpdateStatusMenu;
+ (BOOL) compareInstalledVersion:(NSString*) installedVersionString withUpdateVersion:(NSString*)updateVersionString;
- (void) killUnprivilegedApp:(NSString*)appName;
+ (int) getPIDForProcess:(NSString*) appName;

#pragma mark autolaunching
- (void) removeAutoLaunchOfAppPath:(NSString*)appPath; // deprecated, appPath should look like @"/Applications/WD 2go LiveFile.app"
- (void) removeAutoLaunchOfAppPath:(NSString*)appPath locally:(BOOL) doItForSession; // appPath should look like @"/Applications/WD 2go LiveFile.app"
- (void) removeAutoLaunchOfAppName:(NSString *)appName;  // deprecated, should look like @"WD 2go LiveFile.app"
- (void) removeAutoLaunchOfAppName:(NSString *)appName locally:(BOOL) doItForSession; // deprecated, should look like @"WD 2go LiveFile.app"
- (void) setupAutoLaunchOfAppPath:(NSString*)appPath; // should look like @"/Applications/WD 2go LiveFile.app"
- (void) setupAutoLaunchOfAppPath:(NSString*)appPath locally:(BOOL) doItForSession; // appPath should look like @"/Applications/WD 2go LiveFile.app"
- (void) launchApp:(NSString*)appPath launchFlags:(LSLaunchFlags)launchFlags;
- (void)enableLoginItemWithLoginItemsReference:(LSSharedFileListRef )theLoginItemsRefs ForPath:(NSString *)appPath; // deprecated
- (void)enableLoginItemWithLoginItemsReference:(LSSharedFileListRef )theLoginItemsRefs ForPath:(NSString *)appPath locally:(BOOL) doItForSession;
- (void)disableLoginItemWithLoginItemsReference:(LSSharedFileListRef )theLoginItemsRefs ForPath:(NSString *)appPath; // deprecated
- (void)disableLoginItemWithLoginItemsReference:(LSSharedFileListRef )theLoginItemsRefs ForPath:(NSString *)appPath locally:(BOOL) doItForSession;
- (BOOL)loginItemExistsWithLoginItemReference:(LSSharedFileListRef)theLoginItemsRefs ForPath:(NSString *)appPath;
- (BOOL)loginItemExistsWithLoginItemReference:(LSSharedFileListRef)theLoginItemsRefs WithAppName:(NSString *)appName;
- (void)disableAnyLoginItemWithLoginItemsReference:(LSSharedFileListRef )theLoginItemsRefs WithAppName:(NSString *)appName; // deprecated
- (void)disableAnyLoginItemWithLoginItemsReference:(LSSharedFileListRef )theLoginItemsRefs WithAppName:(NSString *)appName locally:(BOOL) doItForSession;


@end



